@@ -32,6 +32,7 @@
 #include "gnome-print-i18n.h"
 #include "gpa-transport-selector.h"
 #include <libgnomeprint/private/gnome-print-config-private.h>
+#include <libgnomeprint/gnome-print-config.h>
 #include <libgnomeprint/private/gpa-node-private.h>
 #include <libgnomeprint/private/gpa-key.h>
 
@@ -40,13 +41,13 @@ static void gpa_transport_selector_init (GPATransportSelector *selector);
 static void gpa_transport_selector_finalize (GObject *object);
 static gint gpa_transport_selector_construct (GPAWidget *widget);
 
-static void gpa_transport_selector_file_entry_changed_cb       (GtkEntry *entry, GPATransportSelector *ts);
+static void gpa_transport_selector_file_button_clicked_cb       (GtkButton *button, GPATransportSelector *ts);
 static void gpa_transport_selector_custom_entry_changed_cb (GtkEntry *entry, GPATransportSelector *ts);
 static void gpa_transport_selector_node_modified_cb (GPANode *node, guint flags, GPATransportSelector *ts);
 
 static GPAWidgetClass *parent_class;
 
-GtkType
+GType
 gpa_transport_selector_get_type (void)
 {
 	static GType type = 0;
@@ -88,16 +89,19 @@ gpa_transport_selector_init (GPATransportSelector *ts)
 	hbox = (GtkBox *) gtk_hbox_new (FALSE, 4);
 
 	ts->menu = gtk_option_menu_new ();
-	ts->file_entry   = gtk_entry_new ();
+	ts->file_name   = g_strdup ("");
+	ts->file_name_label   = gtk_label_new ("");
+	ts->file_button   = gtk_button_new_from_stock (GTK_STOCK_SAVE_AS);
 	ts->custom_entry = gtk_entry_new ();
 
-	g_signal_connect (G_OBJECT (ts->file_entry), "changed", (GCallback)
-			  gpa_transport_selector_file_entry_changed_cb, ts);
+	g_signal_connect (G_OBJECT (ts->file_button), "clicked", (GCallback)
+			  gpa_transport_selector_file_button_clicked_cb, ts);
 	g_signal_connect (G_OBJECT (ts->custom_entry), "changed", (GCallback)
 			  gpa_transport_selector_custom_entry_changed_cb, ts);
 
 	gtk_box_pack_start (hbox, ts->menu,         FALSE, FALSE, 0);
-	gtk_box_pack_start (hbox, ts->file_entry,   FALSE, FALSE, 0);
+	gtk_box_pack_start (hbox, ts->file_button,   FALSE, FALSE, 0);
+	gtk_box_pack_start (hbox, ts->file_name_label,  FALSE, FALSE, 0);
 	gtk_box_pack_start (hbox, ts->custom_entry, FALSE, FALSE, 0);
 	
 	gtk_container_add (GTK_CONTAINER (ts), GTK_WIDGET (hbox));
@@ -125,28 +129,144 @@ gpa_transport_selector_finalize (GObject *object)
 
 	ts = (GPATransportSelector *) object;
 
+	if (ts->file_selector)
+		gtk_widget_destroy (ts->file_selector);
+	ts->file_selector = NULL;
+
 	gpa_transport_selector_disconnect (ts);
 
 	if (ts->handler_config)
 		g_signal_handler_disconnect (ts->config, ts->handler_config);
 	ts->handler_config = 0;
 	ts->config = NULL;
+	g_free (ts->file_name);
+	ts->file_name = NULL;
 	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
-gpa_transport_selector_file_entry_changed_cb (GtkEntry *entry, GPATransportSelector *ts)
+gpa_transport_selector_file_selector_destroyed_cb (GtkObject *object,
+						   GPATransportSelector *ts)
 {
-	const guchar *text;
+	ts->file_selector = NULL;
+}
 
-	if (ts->updating)
+static void
+gpa_transport_selector_file_selected_cb (GtkWidget *button, GPATransportSelector *ts) 
+{
+	const gchar *selected_filename;
+	gchar *filename;
+	gsize bytes_read;
+	gsize bytes_written;
+
+	selected_filename = gtk_file_selection_get_filename 
+		(GTK_FILE_SELECTION (ts->file_selector));
+	
+	filename = g_filename_to_utf8 (selected_filename, -1, 
+                                       &bytes_read, &bytes_written,NULL);
+
+	if (g_file_test (selected_filename, G_FILE_TEST_IS_DIR)) {
+	/* The following is commented out due to the string freeze */
+	/*	GtkWidget *dialog;
+		dialog = gtk_message_dialog_new 
+			(GTK_WINDOW (ts->file_selector),
+			 GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
+			 GTK_BUTTONS_CLOSE,
+			 _("The specified filename \"%s\" is an existing "
+			   "directory."), filename);
+		
+		g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
+					  G_CALLBACK (gtk_widget_destroy),
+					  GTK_OBJECT (dialog));
+		gtk_widget_show (dialog);
+	*/
+		g_warning ("Selected filename %s is a directory.", filename);
 		return;
-				
-	text = gtk_entry_get_text (entry);
-	ts->updating = TRUE;
-	gpa_node_set_path_value (ts->node, "FileName", text);
-	ts->updating = FALSE;
+	}
+
+	if (g_file_test (selected_filename, G_FILE_TEST_EXISTS)) {
+	/* The following is commented out due to the string freeze */
+	/*	GtkWidget *dialog;
+		gint response;
+
+		dialog = gtk_message_dialog_new 
+			(GTK_WINDOW (ts->file_selector), 
+			 GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
+			 GTK_BUTTONS_YES_NO,
+			 _("Should the file %s be overwritten?"), filename);
+		
+		response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+		gtk_widget_destroy (dialog);
+
+		if (GTK_RESPONSE_YES != response)
+	*/
+		g_warning ("Selected filename %s exists.", filename);
+			return;
+	}
+
+	gpa_node_set_path_value (ts->config, 
+				 "Settings.Transport.Backend.FileName", 
+				 filename);
+	gpa_node_set_path_value (ts->node, "FileName", filename);
+	g_free (ts->file_name);
+	ts->file_name = filename;
+	gtk_label_set_text (GTK_LABEL (ts->file_name_label), ts->file_name);
+}
+
+static void
+gpa_transport_selector_file_button_clicked_cb (GtkButton *button, GPATransportSelector *ts)
+{
+	gchar     *filename;
+	gsize bytes_read;
+	gsize bytes_written;
+	
+	/* Create the selector */
+   
+	ts->file_selector = gtk_file_selection_new 
+		(_("Please specify the location and filename of the output file:"));
+	
+	filename = g_filename_from_utf8 (ts->file_name, -1,
+					 &bytes_read, &bytes_written, NULL);
+	if (filename != NULL) {
+		gtk_file_selection_set_filename 
+			(GTK_FILE_SELECTION(ts->file_selector),
+			 filename);
+		g_free (filename);
+	}
+   
+	g_signal_connect 
+		(GTK_OBJECT(GTK_FILE_SELECTION(ts->file_selector)->ok_button),
+		 "clicked",
+		 G_CALLBACK(gpa_transport_selector_file_selected_cb),
+		 (gpointer) ts);
+	g_signal_connect 
+		(GTK_OBJECT(ts->file_selector),
+		 "destroy",
+		 G_CALLBACK(gpa_transport_selector_file_selector_destroyed_cb),
+		 (gpointer) ts);
+   			   
+	/* Ensure that the dialog box is destroyed when the user clicks a button. */
+	
+	g_signal_connect_swapped 
+		(GTK_OBJECT(GTK_FILE_SELECTION(ts->file_selector)
+			    ->ok_button),
+		 "clicked",
+		 G_CALLBACK(gtk_widget_destroy), 
+		 (gpointer) ts->file_selector); 
+	
+	g_signal_connect_swapped 
+		(GTK_OBJECT(GTK_FILE_SELECTION(ts->file_selector)
+			    ->cancel_button),
+		 "clicked",
+		 G_CALLBACK(gtk_widget_destroy),
+		 (gpointer) ts->file_selector); 
+    
+	gtk_window_set_modal(GTK_WINDOW (ts->file_selector), TRUE);
+	/* Display that dialog */
+	gtk_widget_show (ts->file_selector);
+	
 }
 
 static void
@@ -172,14 +292,20 @@ gpa_transport_selector_update_widgets (GPATransportSelector *ts)
 	filename = gpa_node_get_path_value (ts->config, "Settings.Transport.Backend.FileName");
 	command  = gpa_node_get_path_value (ts->config, "Settings.Transport.Backend.Command");
 	
-	gtk_widget_hide (ts->file_entry);
+	gtk_widget_hide (ts->file_name_label);
+	gtk_widget_hide (ts->file_button);
 	gtk_widget_hide (ts->custom_entry);
 
 	if (backend && !strcmp (backend, "file")) {
 		ts->updating = TRUE;
-		gtk_entry_set_text (GTK_ENTRY (ts->file_entry), filename ? filename : "gnome-print.out");
+		g_free (ts->file_name);
+		ts->file_name = g_strdup (filename ? filename 
+					  : "gnome-print.out");
+		gtk_label_set_text (GTK_LABEL (ts->file_name_label), 
+				    ts->file_name);
 		ts->updating = FALSE;
-		gtk_widget_show (ts->file_entry);
+		gtk_widget_show (ts->file_button);
+		gtk_widget_show (ts->file_name_label);
 	}
 	
 	if (backend && !strcmp (backend, "custom")) {
@@ -219,41 +345,50 @@ gpa_transport_selector_rebuild_menu (GPATransportSelector *ts)
 	menu = gtk_menu_new ();
 	gtk_widget_show (menu);
 
-	option = GPA_KEY (ts->node)->option;
-	def = gpa_node_get_value (ts->node);
-
-	child = gpa_node_get_child (option, NULL);
-	while (child) {
-		const guchar *id;
-		guchar *name;
-
-		name = gpa_node_get_value (child);
-		item = gtk_menu_item_new_with_label (name);
-		g_free (name);
-
-		id = gpa_node_id (child);
-
-		g_signal_connect (G_OBJECT (item), "activate", (GCallback) gpa_transport_selector_item_activate_cb, ts);
-		g_object_set_data_full (G_OBJECT (item), "node", child, (GtkDestroyNotify) gpa_node_unref);
-		gtk_widget_show (item);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		if (GPA_NODE_ID_COMPARE (child, def))
-		    sel = pos;
+	if (ts->node != NULL) {
+		option = GPA_KEY (ts->node)->option;
+		def = gpa_node_get_value (ts->node);
 		
-		pos++;
-		child = gpa_node_get_child (option, child);
+		child = gpa_node_get_child (option, NULL);
+		while (child) {
+			const guchar *id;
+			guchar *name;
+			
+			name = gpa_node_get_value (child);
+			item = gtk_menu_item_new_with_label (name);
+			g_free (name);
+			
+			id = gpa_node_id (child);
+			
+			g_signal_connect 
+				(G_OBJECT (item), "activate", 
+				 (GCallback) 
+				 gpa_transport_selector_item_activate_cb, ts);
+			g_object_set_data_full 
+				(G_OBJECT (item), "node", child, 
+				 (GtkDestroyNotify) gpa_node_unref);
+			gtk_widget_show (item);
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+			if (GPA_NODE_ID_COMPARE (child, def))
+				sel = pos;
+			
+			pos++;
+			child = gpa_node_get_child (option, child);
+		}
+		if (sel == -1) {
+			g_warning ("rebuild_menu_cb, could not set value "
+				   "of %s to %s",
+				   gpa_node_id (option), def);
+			sel = 0;
+		}
 	}
 	if (pos < 1) {
-		item = gtk_menu_item_new_with_label (_("No options are defined"));
+		item = gtk_menu_item_new_with_label 
+			(_("No options are defined"));
 		gtk_widget_set_sensitive (item, FALSE);
 		gtk_widget_show (item);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	}
-	if (sel == -1) {
-		g_warning ("rebuild_menu_cb, could not set value of %s to %s",
-			   gpa_node_id (option), def);
-		sel = 0;
-	}
 
 	gtk_widget_show (menu);
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (ts->menu), menu);
@@ -267,8 +402,11 @@ static void
 gpa_transport_selector_connect (GPATransportSelector *ts)
 {
 	ts->node = gpa_node_lookup (ts->config, "Settings.Transport.Backend");
-	ts->handler = g_signal_connect (G_OBJECT (ts->node), "modified", (GCallback)
-					gpa_transport_selector_node_modified_cb, ts);
+	if (ts->node)
+		ts->handler = g_signal_connect 
+			(G_OBJECT (ts->node), 
+			 "modified", (GCallback)
+			 gpa_transport_selector_node_modified_cb, ts);
 }
 
 static void
